@@ -1,5 +1,5 @@
 //  Advanced System UI
-//  Copyright (c) 2010-2011 Brand Huntsman <http://qzx.com/mail/>
+//  Copyright (c) 2010-2011 Brand Huntsman <brand.huntsman@gmail.com>
 
 //#define CLEAR_KEYS
 
@@ -19,7 +19,8 @@ const char *conf_dim_timeout = "/system/osso/dsm/display/display_dim_timeout";
 const char *conf_volume = "/apps/osso/sound/master_volume";
 const char *conf_screen_autolock1 = "/apps/osso/applet/osso-applet-display/lock_screen";
 const char *conf_screen_autolock2 = "/system/osso/dsm/locks/touchscreen_keypad_autolock_enabled";
-const char *conf_screen_stayslit = "/system/osso/dsm/display/display_on_with_charger";
+const char *conf_screen_stayslit1 = "/apps/osso/applet/osso-applet-display/display_charger";
+const char *conf_screen_stayslit2 = "/system/osso/dsm/display/display_on_with_charger";
 
 const char *conf_brightness_level = "/apps/asui_state/brightness_level";
 const char *conf_theme = "/apps/asui_state/theme";
@@ -31,6 +32,7 @@ GConfClient *gconf_client;
 unsigned cfg_battery_idle_time, cfg_memory_format, cfg_swap_format, cfg_temperature_F;
 // clock
 unsigned cfg_clock_24h, cfg_clock_ampm;
+unsigned cfg_popup_duration, cfg_popup_onmap, cfg_popup_fullscreen;
 // brightness
 unsigned cfg_brightness_enhanced, cfg_brightness_allow_zero;
 // BRV keys
@@ -39,6 +41,7 @@ unsigned cfg_buttons_rotate, cfg_buttons_brightness, cfg_buttons_brightness_step
 unsigned cfg_double_tap, cfg_lock_and_close, cfg_n800_lock_mode, cfg_rotate_and_close;
 unsigned cfg_buttons_dpad_short, cfg_buttons_dpad_long, cfg_buttons_lock1_short, cfg_buttons_lock1_long, cfg_buttons_lock2_short, cfg_buttons_lock2_long;
 unsigned cfg_secure_action, cfg_secure_power_action, cfg_secure_button;
+unsigned cfg_power_unlock_key;
 // user interface
 unsigned cfg_service_size_unit, cfg_short_tap_markers, cfg_start_visible, cfg_threshold_black, cfg_threshold_white;
 
@@ -92,6 +95,9 @@ static const char *conf_temperature_F = "/apps/asui/temperature_F";
 // clock
 static const char *conf_clock_24h = "/apps/asui/clock_24h";
 static const char *conf_clock_ampm = "/apps/asui/clock_ampm";
+static const char *conf_popup_duration = "/apps/asui/popup_duration";
+static const char *conf_popup_onmap = "/apps/asui/popup_onmap";
+static const char *conf_popup_fullscreen = "/apps/asui/popup_fullscreen";
 // brightness
 static const char *conf_brightness_enhanced = "/apps/asui/brightness_enhanced";
 static const char *conf_brightness_allow_zero = "/apps/asui/brightness_allow_zero";
@@ -115,6 +121,7 @@ static const char *conf_buttons_lock2_long = "/apps/asui/buttons_lock2_long";
 static const char *conf_secure_action = "/apps/asui/secure_action";
 static const char *conf_secure_power_action = "/apps/asui/secure_power_action";
 static const char *conf_secure_button = "/apps/asui/secure_button";
+static const char *conf_power_unlock_key = "/apps/asui/power_unlock_key";
 // user interface
 static const char *conf_service_size_unit = "/apps/asui/service_size_unit";
 static const char *conf_short_tap_markers = "/apps/asui/short_tap_markers";
@@ -300,6 +307,9 @@ void config_init( ){
 	// clock
 	gconf_client_unset(gconf_client, conf_clock_24h, NULL);
 	gconf_client_unset(gconf_client, conf_clock_ampm, NULL);
+	gconf_client_unset(gconf_client, conf_popup_duration, NULL);
+	gconf_client_unset(gconf_client, conf_popup_onmap, NULL);
+	gconf_client_unset(gconf_client, conf_popup_fullscreen, NULL);
 	// brightness
 	gconf_client_unset(gconf_client, conf_brightness_enhanced, NULL);
 	gconf_client_unset(gconf_client, conf_brightness_allow_zero, NULL);
@@ -323,6 +333,7 @@ void config_init( ){
 	gconf_client_unset(gconf_client, conf_secure_action, NULL);
 	gconf_client_unset(gconf_client, conf_secure_power_action, NULL);
 	gconf_client_unset(gconf_client, conf_secure_button, NULL);
+	gconf_client_unset(gconf_client, conf_power_unlock_key, NULL);
 	// user interface
 	gconf_client_unset(gconf_client, conf_service_size_unit, NULL);
 	gconf_client_unset(gconf_client, conf_short_tap_markers, NULL);
@@ -404,6 +415,9 @@ void config_reload( ){
 	// clock
 	make_bool(&cfg_clock_24h,				conf_clock_24h,					1,		0);					// default: 1	24 hour clock
 	make_bool(&cfg_clock_ampm,				conf_clock_ampm,				0,		0);					// default: 0	24 hour clock doesn't use this
+	make_uint(&cfg_popup_duration,			conf_popup_duration,			0,		0);					// default: 0	popup diabled
+	make_bool(&cfg_popup_onmap,				conf_popup_onmap,				0,		0);					// default: 0	off
+	make_bool(&cfg_popup_fullscreen,		conf_popup_fullscreen,			0,		0);					// default: 0	small popup window
 	// brightness
 	make_bool(&cfg_brightness_enhanced,		conf_brightness_enhanced,		!is_770,HW_BRIGHTNESS);		// default: 1	all 127 brightness levels
 	make_bool(&cfg_brightness_allow_zero,	conf_brightness_allow_zero,		0,		HW_BRIGHTNESS);		// default: 0	don't allow zero brightness
@@ -427,6 +441,7 @@ void config_reload( ){
 	make_uint(&cfg_secure_action,			conf_secure_action,				2,		0);					// default: 0	lock&blank
 	make_uint(&cfg_secure_power_action,		conf_secure_power_action,		1,		0);					// default: 0	lock&blank
 	make_bool(&cfg_secure_button,			conf_secure_button,				0,		HW_THEME);			// default: 0	don't show secure button
+	make_uint(&cfg_power_unlock_key,		conf_power_unlock_key,			1,		0);					// default: 1	power button + fullscreen key unlocks
 	// user interface
 	make_bool(&cfg_service_size_unit,		conf_service_size_unit,			1,		HW_THEME);			// default: 1	show MB with service sizes
 	make_bool(&cfg_short_tap_markers,		conf_short_tap_markers,			0,		HW_THEME);			// default: 0	don't show short tap markers
@@ -444,6 +459,9 @@ void config_reload( ){
 	if(cfg_memory_format > 3){ cfg_memory_format = 0; conf_set_int(conf_memory_format, 0); }
 	if(cfg_swap_format > 3){ cfg_swap_format = 0; conf_set_int(conf_swap_format, 0); }
 
+	// popup window duration
+	if(cfg_popup_duration > 15){ cfg_popup_duration = 5; conf_set_int(conf_popup_duration, 5); }
+
 	// make sure buttons are correctly bound and don't exceed step values
 	if(cfg_buttons_rotate > 3){ cfg_buttons_rotate = 0; conf_set_int(conf_buttons_rotate, 0); } // disable if invalid
 	if(cfg_buttons_brightness > 3) cfg_buttons_brightness = 0; // disable if invalid
@@ -477,6 +495,7 @@ void config_reload( ){
 	if(cfg_buttons_lock2_long > 4){ cfg_buttons_lock2_long = 0; conf_set_int(conf_buttons_lock2_long, 0); } // disable if invalid
 	if(cfg_secure_action > 3){ cfg_secure_action = 0; conf_set_int(conf_secure_action, 0); } // disable if invalid
 	if(cfg_secure_power_action > 2){ cfg_secure_power_action = 0; conf_set_int(conf_secure_power_action, 0); } // disable if invalid
+	if(cfg_power_unlock_key > 5){ cfg_power_unlock_key = 1; conf_set_int(conf_power_unlock_key, 1); } // set to fullscreen if invalid
 
 // override until 770 is supported
 if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_bool(conf_brightness_enhanced, 0); } // disable
@@ -515,6 +534,8 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 
 	cfg_start_visible = 1; // no power button to map it
 	cfg_clock_24h = 1; // because I like 24 hour
+	cfg_popup_duration = 0;
+	cfg_popup_onmap = 0;
 	cfg_threshold_white = 4000; // must be set otherwise white theme is used
 	cfg_short_tap_markers = 1; // show all markers
 	cfg_double_tap = 1; // require double tap
@@ -544,6 +565,9 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 	// clock
 	debug_write("  cfg_clock_24h = %u", cfg_clock_24h);
 	debug_write("  cfg_clock_ampm = %u", cfg_clock_ampm);
+	debug_write("  cfg_popup_duration = %u", cfg_popup_duration);
+	debug_write("  cfg_popup_onmap = %u", cfg_popup_onmap);
+	debug_write("  cfg_popup_fullscreen = %u", cfg_popup_fullscreen);
 	// brightness
 	debug_write("  cfg_brightness_enhanced = %u", cfg_brightness_enhanced);
 	debug_write("  cfg_brightness_allow_zero = %u", cfg_brightness_allow_zero);
@@ -567,6 +591,7 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 	debug_write("  cfg_secure_action = %u", cfg_secure_action);
 	debug_write("  cfg_secure_power_action = %u", cfg_secure_power_action);
 	debug_write("  cfg_secure_button = %u", cfg_secure_button);
+	debug_write("  cfg_power_unlock_key = %u", cfg_power_unlock_key);
 	// user interface
 	debug_write("  cfg_service_size_unit = %u", cfg_service_size_unit);
 	debug_write("  cfg_short_tap_markers = %u", cfg_short_tap_markers);
@@ -587,6 +612,11 @@ void conf_set_screen_autolock_value( unsigned value ){
 	conf_set_bool(conf_screen_autolock2, value);
 }
 
+void conf_set_screen_stayslit_value( unsigned value ){
+	conf_set_bool(conf_screen_stayslit1, value);
+	conf_set_bool(conf_screen_stayslit2, value);
+}
+
 void config_save_widget( s_widget *widget, unsigned cell, unsigned subcell, unsigned landscape ){
 	if(landscape)
 		widget->cell = (widget->cell&0xff00) | cell<<4 | subcell;
