@@ -6,7 +6,8 @@ extern const char *conf_dim_timeout;
 extern const char *conf_volume;
 extern const char *conf_screen_autolock1;
 extern const char *conf_screen_autolock2;
-extern const char *conf_screen_stayslit;
+extern const char *conf_screen_stayslit1;
+extern const char *conf_screen_stayslit2;
 
 extern const char *conf_brightness_level;
 extern const char *conf_theme;
@@ -18,6 +19,7 @@ extern GConfClient *gconf_client;
 extern unsigned cfg_battery_idle_time, cfg_memory_format, cfg_swap_format, cfg_temperature_F;
 // clock
 extern unsigned cfg_clock_24h, cfg_clock_ampm;
+extern unsigned cfg_popup_duration, cfg_popup_onmap, cfg_popup_fullscreen;
 // brightness
 extern unsigned cfg_brightness_enhanced, cfg_brightness_allow_zero;
 // BRV keys
@@ -26,6 +28,7 @@ extern unsigned cfg_buttons_rotate, cfg_buttons_brightness, cfg_buttons_brightne
 extern unsigned cfg_double_tap, cfg_lock_and_close, cfg_n800_lock_mode, cfg_rotate_and_close;
 extern unsigned cfg_buttons_dpad_short, cfg_buttons_dpad_long, cfg_buttons_lock1_short, cfg_buttons_lock1_long, cfg_buttons_lock2_short, cfg_buttons_lock2_long;
 extern unsigned cfg_secure_action, cfg_secure_power_action, cfg_secure_button;
+extern unsigned cfg_power_unlock_key;
 // user interface
 extern unsigned cfg_service_size_unit, cfg_short_tap_markers, cfg_start_visible, cfg_threshold_black, cfg_threshold_white;
 // state
@@ -86,8 +89,8 @@ void config_reload( );
 #define conf_set_int(key, value) gconf_client_set_int(gconf_client, key, value, (GError **)NULL)
 
 #define conf_get_screen_autolock_value() gconf_client_get_bool(gconf_client, conf_screen_autolock1, (GError **)NULL)
-#define conf_get_screen_stayslit_value() gconf_client_get_bool(gconf_client, conf_screen_stayslit, (GError **)NULL)
+#define conf_get_screen_stayslit_value() gconf_client_get_bool(gconf_client, conf_screen_stayslit1, (GError **)NULL)
 void conf_set_screen_autolock_value( unsigned value );
-#define conf_set_screen_stayslit_value(value) gconf_client_set_bool(gconf_client, conf_screen_stayslit, value, (GError **)NULL)
+void conf_set_screen_stayslit_value( unsigned value );
 
 void config_save_widget( s_widget *widget, unsigned cell, unsigned subcell, unsigned landscape );
