@@ -1,5 +1,5 @@
 //  Advanced System UI
-//  Copyright (c) 2010-2011 Brand Huntsman <http://qzx.com/mail/>
+//  Copyright (c) 2010-2011 Brand Huntsman <brand.huntsman@gmail.com>
 
 // http://maemo.org/api_refs/4.1/dbus-1.0.2/
 // http://dbus.freedesktop.org/doc/dbus/api/html/index.html
@@ -22,9 +22,10 @@
 
 #include "window.h"
 #include "main.h"
- #include "dbus.h"
 #include "config.h"
 #include "hardware.h"
+ #include "dbus.h"
+#include "dsme.h"
 
 ////////////////////////////////////////////////////////////////////////// GLOBALS
 
@@ -53,6 +54,7 @@ static const char *_notification_method_ = "SystemNoteInfoprint"; // method
 //////////////////////////////////////////////////////////////////////////
 
 #ifdef ARCH_armel
+/*
 static void check_dimmed_state( ){
 	if(hw_screen.dimmed && hw_battery.status != BATTERY_DRAINING && conf_get_screen_stayslit_value()){
 		// unset dimmed state if on charger and screen stays lit
@@ -62,6 +64,7 @@ static void check_dimmed_state( ){
 		dbus_send_wakeup(_WAKEUP_REFRESH_);
 	}
 }
+*/
 
 static void set_battery_status( e_battery_status status ){
 	unsigned charging = (status == BATTERY_CHARGING ? 1 : 0);
@@ -74,7 +77,7 @@ static void set_battery_status( e_battery_status status ){
 		if(!charging) hw_query_battery(1); // flush
 	}
 
-	check_dimmed_state();
+//	check_dimmed_state();
 }
 
 static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage *message, void *user_data ){
@@ -149,14 +152,41 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 			if(!strcmp(s1, "ButtonPressed")){
 				if(!strcmp(s2, "power")){
 					debug_write("[DBUS]         power button pressed (blank=%u)", blank_window_open);
-					if(blank_window_open)
+
+					// this signal will turn on the screen, if off
+					dsme_display_state_waiting = 0; // reset blanking pause check
+
+					if(cfg_power_unlock_key && hw_screen.locked){
+						if(hw_screen.visible && current_time - hw_screen.visible_time > 1.0){
+							// turn screen back off
+							mce_lock_and_blank_screen();
+						} else {
+							// secondary key required to unlock screen, keypad remains unlocked
+							hw_lock_screen();
+							hw_unlock_keys();
+							if(!window.is_mapped) window_open_blank(); // open blank to catch keys
+							notify_power_unlock_key();
+						}
+					} else if(blank_window_open)
 						window_close_blank(); // power key caught, close the blank window
 					else
 						dbus_send_wakeup(_WAKEUP_POWER_BUTTON_);
 					return DBUS_HANDLER_RESULT_HANDLED;
 				} else if(!strcmp(s2, "connection")){
 					debug_write("[DBUS]         headphone connection (blank=%u)", blank_window_open);
-					window_close_blank(); // this will turn the screen on, close the blank window
+
+					// this signal will turn on the screen, if off
+					dsme_display_state_waiting = 0; // reset blanking pause check
+
+					if(cfg_power_unlock_key && hw_screen.locked){
+						hw_lock_screen();
+						hw_unlock_keys();
+						if(!window.is_mapped) window_open_blank(); // open blank to catch keys
+						notify_power_unlock_key();
+					} else {
+						if(hw_screen.locked) mce_lock_screen();
+						window_close_blank();
+					}
 					hw_query_headphones();
 					dbus_send_wakeup(_WAKEUP_REFRESH_);
 					return DBUS_HANDLER_RESULT_HANDLED;
@@ -193,11 +223,16 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 		if(dbus_message_get_args(message, &dbus_error, _STRING_, &s1, _END_)){
 			if(!strcmp(s1, "unlocked")){
 				debug_write("[DBUS] signal: com.nokia.mce.signal.tklock_mode_ind (unlock)");
+
+				// the hardware unlock button will send this signal and it will turn on the screen, if off
+				// assume screen is now on and reset blanking pause check
+				dsme_display_state_waiting = 0; // reset blanking pause check
+
 				if(hw_screen.locked){
 					// screen was just unlocked
 					notify_screen_unlocked();
 				}
-				hw_unlock_keypad(); // always make sure keypad is unlocked, even if ASUI doesn't think the screen is locked
+				hw_unlock_keys(); // always make sure keypad is unlocked, even if ASUI doesn't think the screen is locked
 				hw_screen.locked = 0;
 //printf("com.nokia.mce.signal.tklock_mode_ind -- screen UNLOCKED\n");
 			} else if(!strcmp(s1, "locked")){
@@ -206,7 +241,7 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 					// screen was just locked but is still visible
 					notify_screen_locked();
 				}
-				hw_lock_keypad(); // always make sure keypad is locked, even if ASUI doesn't think the screen is unlocked
+				hw_lock_keys(); // always make sure keypad is locked, even if ASUI doesn't think the screen is unlocked
 				hw_screen.locked = 1;
 //printf("com.nokia.mce.signal.tklock_mode_ind -- screen LOCKED\n");
 
@@ -217,6 +252,7 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 		}
 		return DBUS_HANDLER_RESULT_HANDLED;
 
+/*
 	} else if(dbus_message_is_signal(message, "com.nokia.mce.signal", "display_status_ind")){
 		char *s1;
 
@@ -245,6 +281,7 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 					original_screen_stayslit = 0;
 					check_dimmed_state();
 				}
+				window_open_popup();
 			} else if(hw_screen.visible && !visible){
 				// screen was just turned off
 				if(!hw_screen.locked){
@@ -255,10 +292,12 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 						click.enabled = 0; // ignore first screen tap when unblanked
 					}
 				}
+				window_close_popup();
 			}
 			hw_screen.visible = visible;
 			// don't need to wakeup main loop for this, dbus unlock signal or X screen/key events will wake it
 
+
 			if(hw_screen.dimmed != dimmed){
 				if(!dimmed || !(hw_battery.status != BATTERY_DRAINING && conf_get_screen_stayslit_value())){
 					// ignore dimmed signal if on charger and screen stays lit
@@ -270,6 +309,7 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 			}
 		}
 		return DBUS_HANDLER_RESULT_HANDLED;
+*/
 
 	} else if(dbus_message_is_signal(message, "com.nokia.mce.signal", "devicelock_mode_ind")){
 		char *s1;
@@ -360,6 +400,9 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 		debug_write("[DBUS] signal: com.nokia.bme.signal.charger_connected");
 //printf("com.nokia.bme.signal.charger_connected\n");
 
+		// this signal will turn on the screen, if off
+		dsme_display_state_waiting = 0; // reset blanking pause check
+
 		if(hw_screen.locked){
 			// lock&blank doesn't properly lock the screen and connecting charger will unlock it
 			// without sending an unlock signal
@@ -380,6 +423,9 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 		debug_write("[DBUS] signal: com.nokia.bme.signal.charger_disconnected");
 //printf("com.nokia.bme.signal.charger_disconnected\n");
 
+		// this signal will turn on the screen, if off
+		dsme_display_state_waiting = 0; // reset blanking pause check
+
 		if(hw_battery.status != BATTERY_DRAINING){
 			hw_battery.status = BATTERY_DRAINING;
 			dbus_send_charging_status_signal();
@@ -834,6 +880,31 @@ void notify_screen_unlocked( ){
 	#endif
 }
 
+void notify_power_unlock_key( ){
+	#ifdef ARCH_armel
+	if(cfg_power_unlock_key){
+		dbus_send_session_method(_notification_service_, _notification_path_, _notification_interface_, _notification_method_,
+									_STRING_, &power_unlock_messages[cfg_power_unlock_key], _END_);
+		mce_dim_screen();
+	}
+	#endif
+}
+
+void notify_blanking_pause1( ){
+	#ifdef ARCH_armel
+	char *paused_message   = "An application may have PAUSED screen blanking";
+	dbus_send_session_method(_notification_service_, _notification_path_, _notification_interface_, _notification_method_, _STRING_, &paused_message, _END_);
+	#endif
+}
+void notify_blanking_pause2( ){
+	#ifdef ARCH_armel
+	char *locked_message   = "Screen is LOCKED and should dim in less than 60 seconds";
+	char *unlocked_message = "Screen is UNLOCKED and should dim in less than 60 seconds";
+	dbus_send_session_method(_notification_service_, _notification_path_, _notification_interface_, _notification_method_,
+								_STRING_, (hw_screen.locked ? &locked_message : &unlocked_message), _END_);
+	#endif
+}
+
 ////////////////////////////////////////////////////////////////////////// STATUSBAR APPLET SIGNALS
 
 void dbus_send_charging_status_signal( ){
