@@ -45,6 +45,9 @@ void dbus_send_session_method( const char *destination, const char *path, const
 
 void notify_screen_locked( );
 void notify_screen_unlocked( );
+void notify_power_unlock_key( );
+void notify_blanking_pause1( );
+void notify_blanking_pause2( );
 
 void dbus_send_charging_status_signal( );
 void dbus_send_battery_capacity_signal( );
