@@ -1,5 +1,5 @@
 //	Advanced System UI
-//	Copyright (c) 2010-2011 Brand Huntsman <http://qzx.com/mail/>
+//	Copyright (c) 2010-2011 Brand Huntsman <brand.huntsman@gmail.com>
 //
 //	backlight code ported from advanced-backlight
 //	(C) 2008 Jonas Hurrelmann <j@outpo.st>.
@@ -23,9 +23,9 @@
 
 #include "window.h"
 #include "main.h"
- #include "hardware.h"
 #include "config.h"
 #include "services.h"
+ #include "hardware.h"
 #include "dbus.h"
 #include "dsme.h"
 #include "secure.h"
@@ -243,6 +243,8 @@ static const char *_mce_get_devicelock_mode_ = "get_devicelock_mode"; // method
 static const char *_mce_devlock_callback_ = "devlock_callback"; // method
 static const char *_mce_device_LOCKED_ = "locked";
 
+#define TOUCHSCREEN_CONTROL "/sys/devices/platform/omap2_mcspi.1/spi1.0/disable_ts"
+
 #define KEYPAD_CONTROL_N800 "/sys/devices/platform/omap2_mcspi.1/spi1.0/disable_kp"
 #define KEYPAD_CONTROL_N810 "/sys/devices/platform/i2c_omap.2/i2c-0/0-0045/disable_kp"
 static const char *keypad_control;
@@ -290,8 +292,14 @@ static void mce_get_tklock_callback( DBusPendingCall *pending, void *user_data )
 		hw_screen.locked = (strcmp(mode, _mce_tklock_LOCKED_) ? 0 : 1);
 		if(sl_ready) sl_init(); else sl_ready++;
 		hw_initialized &= ~HWIS_MCE_LOCK;
-		if(hw_screen.locked)
-			hw_lock_keypad(); // lock the keypad
+		unsigned ts_locked = get_file_value(TOUCHSCREEN_CONTROL);
+		unsigned kp_locked = get_file_value(keypad_control);
+		if(cfg_power_unlock_key && ts_locked && !kp_locked){
+			if(!cfg_start_visible) window_open_blank();
+			notify_power_unlock_key();
+		}
+//		else if(hw_screen.locked)
+//			hw_lock_keys(); // lock the keypad
 		debug_write("[DBUS] callback: mce_get_tklock_callback (locked = %u)", hw_screen.locked);
 	}
 	dbus_message_unref(message);
@@ -350,7 +358,7 @@ void mce_init( ){
 
 	#ifdef ARCH_armel
 	keypad_control = (file_exists(KEYPAD_CONTROL_N800) ? KEYPAD_CONTROL_N800 : KEYPAD_CONTROL_N810);
-	hw_unlock_keypad(); // unlock the keypad to match the current hw_screen.locked state, will be relocked by mce_get_tklock_callback()
+//	hw_unlock_keys(); // unlock the keypad to match the current hw_screen.locked state, will be relocked by mce_get_tklock_callback()
 
 	if(!use_minimal_ui){
 		hw_initialized |= HWIS_MCE_DISPLAY|HWIS_MCE_LOCK|HWIS_MCE_FLIGHT|HWIS_MCE_DEVICELOCK;
@@ -365,6 +373,16 @@ void mce_init( ){
 
 ////////////////////
 
+void mce_dim_screen( ){
+	#ifdef ARCH_armel
+	if(hw_battery.status == BATTERY_DRAINING || !conf_get_screen_stayslit_value()){
+		dsme_allow_blanking(); // make sure screen isn't paused
+		dbus_send_method(_mce_service_, _mce_path_, _mce_interface_, "req_display_state_dim", _END_);
+		dsme_expect_display_state(DSME_DISPLAY_DIM);
+	}
+	#endif
+}
+
 void mce_blank_screen( ){
 	#ifdef ARCH_armel
 	unsigned screen_autolock = conf_get_screen_autolock_value();
@@ -388,6 +406,7 @@ void mce_blank_screen( ){
 
 	// blank screen
 	dbus_send_method(_mce_service_, _mce_path_, _mce_interface_, _mce_req_display_state_off_, _END_);
+	dsme_expect_display_state(DSME_DISPLAY_OFF);
 	#endif
 }
 
@@ -414,6 +433,7 @@ void mce_lock_and_blank_screen( ){
 
 	// lock and blank screen
 	dbus_send_method(_mce_service_, _mce_path_, _mce_interface_, _mce_req_display_state_off_, _END_);
+	dsme_expect_display_state(DSME_DISPLAY_OFF);
 	#endif
 }
 
@@ -423,6 +443,8 @@ void mce_lock_screen( ){
 	if(hw_battery.status == BATTERY_DRAINING || !conf_get_screen_stayslit_value())
 		dsme_allow_blanking(); // make sure screen isn't paused
 	dbus_send_method(_mce_service_, _mce_path_, _mce_interface_, _mce_req_tklock_mode_change_, _STRING_, &_mce_tklock_LOCKED_, _END_);
+	if(hw_battery.status == BATTERY_DRAINING || !conf_get_screen_stayslit_value())
+		dsme_expect_display_state(DSME_DISPLAY_DIM);
 	#endif
 }
 
@@ -433,7 +455,19 @@ void mce_unlock_screen( ){
 	#endif
 }
 
-void hw_lock_keypad( ){
+void hw_lock_screen( ){
+	#ifdef ARCH_armel
+	FILE *out = fopen(TOUCHSCREEN_CONTROL, "w");
+	if(out != NULL){
+		fputc('1', out);
+		fclose(out);
+	} else {
+		debug_write("failed to open file needed to lock screen");
+	}
+	#endif
+}
+
+void hw_lock_keys( ){
 	#ifdef ARCH_armel
 	FILE *out = fopen(keypad_control, "w");
 	if(out != NULL){
@@ -445,7 +479,7 @@ void hw_lock_keypad( ){
 	#endif
 }
 
-void hw_unlock_keypad( ){
+void hw_unlock_keys( ){
 	#ifdef ARCH_armel
 	FILE *out = fopen(keypad_control, "w");
 	if(out != NULL){
@@ -883,6 +917,8 @@ static void hw_query_cpu_usage( ){
 
 // http://bluez.cvs.sourceforge.net/viewvc/bluez/utils/hcid/dbus-api.txt
 // http://bluez.cvs.sourceforge.net/viewvc/bluez/utils/audio/audio-api.txt
+// http://maemo.org/maemo_release_documentation/maemo4.1.x/node10.html
+// http://wiki.maemo.org/Documentation/Maemo_5_Developer_Guide/Using_Connectivity_Components/Maemo_Connectivity#Bluetooth_D-Bus_UI_dialogs
 
 #ifdef ARCH_armel
 static const char *_bluez_service_ = "org.bluez";
@@ -1096,6 +1132,7 @@ void hw_bluetooth_open_search( ){
 // http://maemo.org/api_refs/4.0/gpsbt/group__gpsbt__api.html
 // http://andrew.triumf.ca/N810/gpstest/
 // http://andrew.daviel.org/N810-FAQ.html#gpsd
+// http://maemo.org/maemo_release_documentation/maemo4.1.x/node10.html
 
 // TODO:
 
@@ -1103,6 +1140,7 @@ void hw_bluetooth_open_search( ){
 
 // http://maemo.org/api_refs/5.0/beta/icd2/group__dbus__api.html
 // http://maemo.org/api_refs/4.1/libconic-0.16/
+// http://maemo.org/maemo_release_documentation/maemo4.1.x/node10.html
 
 #ifdef ARCH_armel
 //#include <conic/coniciap.h>
