@@ -146,12 +146,14 @@ void hw_query_brightness( );
 void hw_query_brightness_with_delay( );
 void hw_set_brightness( unsigned level, unsigned update );
 
+void mce_dim_screen( );
 void mce_blank_screen( );
 void mce_lock_and_blank_screen( );
 void mce_lock_screen( );
 void mce_unlock_screen( );
-void hw_lock_keypad( );
-void hw_unlock_keypad( );
+void hw_lock_screen( );
+void hw_lock_keys( );
+void hw_unlock_keys( );
 void mce_normal_mode( );
 void mce_flight_mode( );
 void mce_lock_device( );
