@@ -1,12 +1,12 @@
 //	Advanced System UI
-//	Copyright (c) 2010-2011 Brand Huntsman <http://qzx.com/mail/>
+//	Copyright (c) 2010-2011 Brand Huntsman <brand.huntsman@gmail.com>
 //
 //	device model detection code ported from advanced-backlight
 //	(C) 2008 Jonas Hurrelmann <j@outpo.st>.
 //	(C) 2008 Adam Harwell <aharwell@trinity.edu>.
 
 // write debug messages to stdout
-//#define DEBUG_TO_STDOUT
+#define DEBUG_TO_STDOUT
 
 // milliseconds
 #define DOUBLE_TAP_DURATION 500
@@ -45,7 +45,7 @@
 #include "window.h"
 #include <X11/extensions/Xrandr.h>
 
-#include "main.h"
+ #include "main.h"
 #include "draw.h"
 #include "text.h"
 #include "config.h"
@@ -101,11 +101,18 @@ unsigned primary_widgets, widget_edit_mode;
 s_selected_cell selected_cell;
 unsigned brightness_widget_page, volume_widget_page, battery_widget_page;
 unsigned flightmode_button_page, keyrepeat_button_page, screenstayslit_button_page, screenautolock_button_page;
+		// new *_page variables must be added to window_map() and dialog_reset()
 
 // idle timer to autolock device
 s_timer idle_timer;
 unsigned is_idle;
 
+unsigned power_unlock_keys[] = {0, KEYCODE_FULLSCREEN, KEYCODE_HOME, KEYCODE_ESCAPE, KEYCODE_DPAD_CENTER, KEYCODE_MENU};
+char *power_unlock_messages[] = {"",
+	"Press FULLSCREEN key to unlock", "Press HOME key to unlock", "Press ESCAPE key to unlock",
+	"Press DPAD CENTER key to unlock", "Press MENU key to unlock"
+};
+
 ////////////////////////////////////////////////////////////////////////// GET MICRO TIME
 
 double get_utime( ){
@@ -1137,7 +1144,7 @@ static void segfault_handler( int nothing ){
 	exit(EXIT_FAILURE);
 }
 
-static unsigned ready_to_read( int fd ){
+unsigned ready_to_read( int fd ){
 	struct timeval timeout;
 	fd_set set;
 
@@ -1299,7 +1306,7 @@ int main( int argc, char *argv[] ){
 
 	int x_fd = ConnectionNumber(window.display);
 	unsigned main_refresh_all = 0, infinite_timeout = (!window.is_mapped || !hw_screen.visible ? 1 : 0);
-	double next_timer_duration = MAIN_REFRESH_PERIOD;
+	double next_timer_duration = MAIN_REFRESH_PERIOD, last_focus_grab = 0.0;
 
 	s_timer refresh_timer, button_timer, key_timer, brightness_timer;
 	set_timer(&refresh_timer, MAIN_REFRESH_PERIOD);
@@ -1325,6 +1332,7 @@ int main( int argc, char *argv[] ){
 		FD_SET(dbus_fd, &set);
 		FD_SET(x_fd, &set);
 		if(blank_fd){ FD_SET(blank_fd, &set); highest_fd = (blank_fd > highest_fd ? blank_fd : highest_fd); }
+		if(popup_fd){ FD_SET(popup_fd, &set); highest_fd = (popup_fd > highest_fd ? popup_fd : highest_fd); }
 		if(!infinite_timeout){
 			if(next_timer_duration > 0.0){
 				// calculate timeout for next refresh
@@ -1349,6 +1357,9 @@ int main( int argc, char *argv[] ){
 		// catch X events for blank window
 		if(blank_fd)
 			if(window_blank_events()) goto exit;
+		// catch X events for popup window
+		if(popup_fd)
+			window_popup_events();
 
 		// catch X events
 		debug_write("any X events?");
@@ -1370,13 +1381,17 @@ int main( int argc, char *argv[] ){
 			case ButtonPress:
 				// e.xbutton.x, e.xbutton.y
 
+				// this event will turn on the screen, if off
+				dsme_display_state_waiting = 0; // reset blanking pause check
+
 				if(!click.enabled && hw_screen.visible && !use_minimal_ui)
 					if(current_time - hw_screen.visible_time > 0.5)
 						click.enabled = 1; // screen has been on for more than 500ms, allow this tap
 
 				click.button_down = 1;
+				click.popup_window = popup_window_open;
 
-				if(click.enabled){
+				if(click.enabled && !popup_window_open){
 					click.type = CLICK_DOWN;
 					click.long_wait = current_time; // long press hasn't been highlighted yet
 					click.timestamp = (unsigned)e.xbutton.time;
@@ -1396,7 +1411,7 @@ int main( int argc, char *argv[] ){
 			case MotionNotify:
 				// e.xmotion.x, e.xmotion.y
 
-				if(click.enabled){
+				if(click.enabled && !click.popup_window){
 					if(click.motion_x != e.xmotion.x || click.motion_y != e.xmotion.y){
 						click.type = CLICK_MOTION;
 						click.offset_x = e.xmotion.x - click.start_x;
@@ -1417,7 +1432,11 @@ int main( int argc, char *argv[] ){
 			case ButtonRelease:
 				// e.xbutton.x, e.xbutton.y
 
-				if(click.enabled){
+				if(click.popup_window){
+
+					window_close_popup();
+
+				} else if(click.enabled){
 
 					if(e.xbutton.x > click.start_x+DRAG_DEVIATION || e.xbutton.x < click.start_x-DRAG_DEVIATION
 					|| e.xbutton.y > click.start_y+DRAG_DEVIATION || e.xbutton.y < click.start_y-DRAG_DEVIATION){
@@ -1452,13 +1471,17 @@ int main( int argc, char *argv[] ){
 			case KeyPress:
 				// e.xkey.keycode
 
+				// this event will turn on the screen, if off
+				dsme_display_state_waiting = 0; // reset blanking pause check
+
 				if(!click.enabled && hw_screen.visible && !use_minimal_ui)
 					if(current_time - hw_screen.visible_time > 0.5)
 						click.enabled = 1; // screen has been on for more than 500ms, allow this key
 
 				keypress.key_down = 1;
+				keypress.popup_window = popup_window_open;
 
-				if(click.enabled){
+				if(click.enabled && !(cfg_power_unlock_key && hw_screen.locked) && !popup_window_open){
 					keypress.type = KEY_DOWN;
 					keypress.timestamp = (unsigned)e.xkey.time;
 					keypress.keycode = e.xkey.keycode;
@@ -1475,7 +1498,18 @@ int main( int argc, char *argv[] ){
 			case KeyRelease:
 				// e.xkey.keycode
 
-				if(click.enabled){
+				if(cfg_power_unlock_key && hw_screen.locked){
+
+					if(e.xkey.keycode == power_unlock_keys[cfg_power_unlock_key])
+						mce_unlock_screen();
+					else
+						notify_power_unlock_key();
+
+				} else if(keypress.popup_window){
+
+					window_close_popup();
+
+				} else if(click.enabled){
 
 					if(keypress.timestamp + (unsigned)(LONG_PRESS_DURATION*1000) < (unsigned)e.xkey.time)
 						keypress.type = KEY_LONG;
@@ -1508,10 +1542,12 @@ int main( int argc, char *argv[] ){
 					XGrabPointer(window.display, window.window, False, ButtonPressMask|ButtonMotionMask|ButtonReleaseMask,
 									GrabModeAsync, GrabModeAsync, window.window, None,CurrentTime);
 					window.has_focus = 1;
+					last_focus_grab = current_time;
 				}
 				break;
 			case FocusOut:			// 10
-				if(window.is_mapped && window.has_focus && !use_minimal_ui){
+				if(window.is_mapped && window.has_focus && !use_minimal_ui && current_time - last_focus_grab > 0.5){
+					// half second delay since last grab to prevent focus in/out loop
 					// release keyboard (home key) when bluetooth or wifi dialog opens
 					XUngrabKeyboard(window.display, CurrentTime);
 					XUngrabPointer(window.display, CurrentTime);
@@ -1539,14 +1575,12 @@ int main( int argc, char *argv[] ){
 				break;
 			default:
 				if(XRRUpdateConfiguration(&e)){
-					if(e.type - window.xrr_event_base == RRScreenChangeNotify){ // 90
+					if(e.type - window.xrr_event_base == RRScreenChangeNotify) // 90
 						update_orientation(&e);
-					} else {
+					else
 						debug_write("unknown RR event %d", e.type - window.xrr_event_base);
-					}
-				} else {
+				} else
 					debug_write("unknown event %d", e.type);
-				}
 			}
 		}
 
@@ -1714,7 +1748,7 @@ int main( int argc, char *argv[] ){
 
 		debug_write("mapped (asui=%u, blank=%u) -- display (on=%u, on_time=%.6f)",
 						window.is_mapped, blank_window_open, hw_screen.visible, current_time - hw_screen.visible_time);
-		if(blank_window_open && hw_screen.visible)
+		if(blank_window_open && hw_screen.visible && !(cfg_power_unlock_key && hw_screen.locked))
 			if(current_time - hw_screen.visible_time > 1.0)
 				window_close_blank(); // screen has been on for more than one second, close the blank window
 
@@ -1725,7 +1759,8 @@ int main( int argc, char *argv[] ){
 		if(current_dialog_draw != NULL && window.is_mapped && hw_screen.visible)
 			hw_query_als();
 
-		infinite_timeout = ((!window.is_mapped && !blank_window_open) || !hw_screen.visible || current_dialog_redraw == DLG_REDRAW_OFF ? 1 : 0);
+		infinite_timeout =
+			((!window.is_mapped && !blank_window_open && !popup_window_open) || !hw_screen.visible || current_dialog_redraw == DLG_REDRAW_OFF ? 1 : 0);
 
 		// auto refresh when display is lit, window is mapped and dialog has a refresh timer
 		if(!infinite_timeout){
@@ -1733,6 +1768,13 @@ int main( int argc, char *argv[] ){
 			debug_write("process timers");
 
 			next_timer_duration = (blank_window_open ? 1.0 : 0.0); // poll every second if the blank window is open
+			if(popup_window_open){
+				double popup_remaining_duration = cfg_popup_duration - (current_time - popup_window_open_time);
+				if(popup_remaining_duration <= 0.0)
+					window_close_popup();
+				else if(next_timer_duration == 0.0 || popup_remaining_duration < next_timer_duration)
+					next_timer_duration = popup_remaining_duration;
+			}
 
 			// secure dialog is always active when device is code locked (fullscreen clock and flashlight can also be active)
 			if(hw_device_locked && !use_minimal_ui
@@ -1820,7 +1862,7 @@ int main( int argc, char *argv[] ){
 			}
 
 			// BUTTON DOWN
-			if(click.enabled && click.button_down){
+			if(click.enabled && click.button_down && !click.popup_window){
 				if(is_timer_ready(&button_timer)){
 					// timer is needed to periodically check for ButtonRelease
 					// and to handle auto-repeat and long press highlighting
@@ -1858,7 +1900,7 @@ int main( int argc, char *argv[] ){
 			}
 
 			// KEY DOWN
-			if(click.enabled && keypress.key_down){
+			if(click.enabled && keypress.key_down && !keypress.popup_window){
 				if(is_timer_ready(&key_timer)){
 					// timer is needed to periodically check for KeyRelease
 					// and to handle auto-repeat and long press highlighting
@@ -1906,6 +1948,12 @@ int main( int argc, char *argv[] ){
 
 		}
 
+		if(dsme_display_state_waiting){
+			double dsme_remaining = dsme_check_display_state();
+			if(dsme_remaining > 0.0 && dsme_remaining < next_timer_duration)
+				next_timer_duration = dsme_remaining;
+		}
+
 		// refresh on dbus signals and method returns
 			// all dialogs should change on HW_THEME
 			// cpu policy dialog needs HW_CPU
