@@ -1,5 +1,5 @@
 //	Advanced System UI
-//	Copyright (c) 2010-2011 Brand Huntsman <http://qzx.com/mail/>
+//	Copyright (c) 2010-2011 Brand Huntsman <brand.huntsman@gmail.com>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -14,8 +14,8 @@
 
 #include "main.h"
 #include "draw.h"
- #include "services.h"
 #include "config.h"
+ #include "services.h"
 #include "hardware.h"
 
 ////////////////////////////////////////////////////////////////////////// GLOBALS
@@ -613,8 +613,8 @@ void svc_init_audio( e_service_action action ){
 		if(svc_audio_status){
 			if(svc_audio_ENGINE) system("/etc/init.d/mediaplayer-daemon stop");
 			if(svc_audio_MULTIMEDIA) system("/etc/init.d/multimediad stop");
-			if(svc_audio_SERVER) system("/etc/osso-af-init/osso-media-server.sh stop");		// this script shouldn't be called directly
-//			if(svc_audio_ESD) system("/etc/init.d/esd stop");								// won't stop
+			if(svc_audio_SERVER) system("/etc/osso-af-init/osso-media-server.sh stop");
+//			if(svc_audio_ESD) system("/etc/init.d/esd stop");								// won't stop, sound applet restarts it
 //			if(svc_audio_DSP) system("/etc/init.d/dsp-init stop");							// won't stop
 			changed = 1;
 		}
@@ -623,7 +623,7 @@ void svc_init_audio( e_service_action action ){
 		if(svc_audio_status != 2){
 			if(!svc_audio_DSP) system("/etc/init.d/dsp-init start");
 			if(!svc_audio_ESD) system("/etc/init.d/esd start");
-			if(!svc_audio_SERVER) system("/etc/osso-af-init/osso-media-server.sh start");	// this script shouldn't be called directly
+			if(!svc_audio_SERVER) system("/etc/osso-af-init/osso-media-server.sh start");	// will only start when launched as root, sudo doesn't work
 			if(!svc_audio_MULTIMEDIA) system("/etc/init.d/multimediad start");
 			if(!svc_audio_ENGINE) system("/etc/init.d/mediaplayer-daemon start");
 			changed = 1;
