@@ -1,5 +1,5 @@
 //	Advanced System UI
-//	Copyright (c) 2010-2011 Brand Huntsman <http://qzx.com/mail/>
+//	Copyright (c) 2010-2011 Brand Huntsman <brand.huntsman@gmail.com>
 
 // seconds (double)
 #define INITIAL_REFRESH_PERIOD 1.0
@@ -24,8 +24,7 @@
 #include "services.h"
 #include "hardware.h"
 #include "dialog.h"
-
-#include "widget_battery.h"
+ #include "widget_battery.h"
 
 //////////////////////////////////////////////////////////////////////////
 
@@ -87,9 +86,12 @@ static int battery_widget_x, battery_widget_y;
 void draw_battery_widget( unsigned force, int x, int y ){
 	if(force){
 		// background for all widgets is cleared by draw_window()
-		battery_widget_page = primary_widgets;
-		battery_widget_x = x;
-		battery_widget_y = y;
+		if(!drawing_popup_window){
+			// don't save values if drawing the popup window
+			battery_widget_page = primary_widgets;
+			battery_widget_x = x;
+			battery_widget_y = y;
+		}
 	} else if(hw_device_locked && (hw_changes & DEVLOCK_BATTERY_FLAGS)){
 		x = battery_widget_x;
 		y = battery_widget_y;
@@ -186,7 +188,7 @@ void draw_battery_widget( unsigned force, int x, int y ){
 			(hw_cpu.usage > 50 ? red_status_color : (hw_cpu.usage ? yellow_status_color : fg2_color)),
 			x+246, y+92, ALIGN_CENTER, ALIGN_BOTTOM, 0);
 
-		draw_tap_markers(x+201, y, 99, 1, hw_swap.total); // short tap only (long tap if swap is enabled)
+		if(!drawing_popup_window) draw_tap_markers(x+201, y, 99, 1, hw_swap.total); // short tap only (long tap if swap is enabled)
 
 		x_set_color(line_color);
 		x_draw_line(x+292, y, x+292, y+99);
@@ -209,7 +211,7 @@ void draw_battery_widget( unsigned force, int x, int y ){
 			draw_text(hw_cpu.current_governor->name, 14, fg1_color, x+380, y+45, ALIGN_RIGHT, ALIGN_MIDDLE, 0);
 		}
 
-		draw_tap_markers(x+293, y, 99, 0, 1); // long tap only
+		if(!drawing_popup_window) draw_tap_markers(x+293, y, 99, 0, 1); // long tap only
 	}
 	{
 		// temperature
