@@ -1,5 +1,5 @@
 //	Advanced System UI
-//	Copyright (c) 2010-2011 Brand Huntsman <http://qzx.com/mail/>
+//	Copyright (c) 2010-2011 Brand Huntsman <brand.huntsman@gmail.com>
 //
 //	XRR initialization and rotation code ported from advanced-backlight
 //	(C) 2008 Jonas Hurrelmann <j@outpo.st>.
@@ -13,7 +13,7 @@
 
 #define APPNAME "Advanced SystemUI"
 
-#include "window.h"
+ #include "window.h"
 
 #include <X11/Xproto.h>
 #include <X11/Xatom.h>
@@ -28,6 +28,7 @@
 #include "services.h"
 #include "hardware.h"
 #include "dialog.h"
+#include "dbus.h"
 
 #include "images/button_toflightmode.xpm"
 #include "images/button_tonormalmode.xpm"
@@ -37,6 +38,13 @@
 #include "images/icon_brightness.xpm"
 #include "images/icon_volume.xpm"
 
+// draw function for popup window (exclusion)
+#include "flashlight.h"
+// draw function for popup window
+#include "widget_battery.h"
+// draw function for popup window
+#include "widget_clock.h"
+
 ////////////////////////////////////////////////////////////////////////// GLOBALS
 
 s_window window;
@@ -53,8 +61,9 @@ s_image im_volume, im_brightness;
 
 unsigned black_theme;
 
-unsigned blank_window_open;
-int blank_fd;
+int blank_fd, popup_fd;
+unsigned blank_window_open, popup_window_open, drawing_popup_window;
+double popup_window_open_time;
 
 //////////////////////////////////////////////////////////////////////////
 
@@ -336,10 +345,13 @@ static int window_create( unsigned first_time ){
 	return 1;
 }
 
-static double last_orientation_check;
+static double last_orientation_check, popup_window_close_time;
 
 int window_init( ){
-	blank_window_open = 0;
+	blank_fd = popup_fd = 0;
+	blank_window_open = popup_window_open = 0;
+	drawing_popup_window = 0;
+	popup_window_close_time = current_time;
 
 	last_orientation_check = 0.0;
 
@@ -409,8 +421,8 @@ void window_auto_set_theme( unsigned initialize ){
 void window_map( ){
 	debug_write("window_map()");
 
-	if(blank_window_open)
-		window_close_blank();
+	window_close_blank();
+	window_close_popup();
 
 	// map the window
 	{
@@ -454,11 +466,17 @@ void window_map( ){
 	window.is_mapped = 1;
 
 	draw_window(1); // force
+
+	// open popup window if it has been more than 10 seconds since it was last closed
+	if(cfg_popup_onmap && current_time - popup_window_close_time > 10.0)
+		window_open_popup();
 }
 
 void window_unmap( ){
 	debug_write("window_unmap()");
 
+	window_close_popup();
+
 	// close dialog if open -- this is needed to reset anything it did to the main loop (the process viewer refresh)
 	dialog_reset();
 
@@ -760,6 +778,9 @@ void update_orientation( XEvent *event ){
 
 //		if(was_mapped)
 //			window_map();
+
+		// close the popup window
+		window_close_popup();
 	} else if(window.is_mapped)
 		draw_window(1); // force to get new rotation icons
 }
@@ -868,17 +889,18 @@ void window_open_blank( ){
 		XFreeGC(blank_display, blank_gc);
 		XDestroyWindow(blank_display, blank_window);
 
-		x_flush();
+		XFlush(blank_display);
 
 		XSetCloseDownMode(blank_display, DestroyAll);
 		XCloseDisplay(blank_display);
 	} else {
 		XGrabKeyboard(blank_display, blank_window, False, GrabModeAsync, GrabModeAsync, CurrentTime);
 
-		x_flush();
+		XFlush(blank_display);
 
 		blank_fd = ConnectionNumber(blank_display);
 		blank_window_open = 1;
+		dbus_send_wakeup(_WAKEUP_REFRESH_); // add blank_fd to main loop
 
 		debug_write("    blank window ready");
 	}
@@ -897,7 +919,7 @@ void window_close_blank( ){
 		XUngrabPointer(blank_display, CurrentTime);
 		XUngrabKeyboard(blank_display, CurrentTime);
 
-		x_flush();
+		XFlush(blank_display);
 
 		XSetCloseDownMode(blank_display, DestroyAll);
 		XCloseDisplay(blank_display);
@@ -917,7 +939,7 @@ unsigned window_blank_events( ){
 		XEvent e;
 
 		XNextEvent(blank_display, &e);
-		debug_write("    event %d", e.type);
+		debug_write("    blank event %d", e.type);
 
 		runaway_event_counter++;
 		if(runaway_event_counter > MAX_RUNAWAY_EVENTS){ debug_write("stuck in a runaway event loop (>%d), killing ASUI", MAX_RUNAWAY_EVENTS); return 1; }
@@ -935,9 +957,19 @@ unsigned window_blank_events( ){
 			// ignore keys from the blank window
 			break;
 		case KeyRelease:
-			// close the blank window
-			window_close_blank();
-			return 0; // get out of this loop or it will segfault
+			if(cfg_power_unlock_key && hw_screen.locked){
+				if(e.xkey.keycode == power_unlock_keys[cfg_power_unlock_key]){
+					window_close_blank();
+					mce_unlock_screen();
+					return 0; // get out of this loop or it will segfault
+				} else
+					notify_power_unlock_key();
+			} else {
+				// close the blank window
+				window_close_blank();
+				return 0; // get out of this loop or it will segfault
+			}
+			break;
 		case NoExpose:			// 14
 		case DestroyNotify:		// 17
 		case UnmapNotify:		// 18
@@ -949,10 +981,221 @@ unsigned window_blank_events( ){
 			// ignore
 			break;
 		default:
-			debug_write("unknown event %d", e.type);
+			debug_write("unknown blank event %d", e.type);
 			break;
 		}
 	}
 
 	return 0;
 }
+
+////////////////////////////////////////////////////////////////////////// POPUP CLOCK WINDOW
+
+#define POPUP_WIDTH 410
+#define POPUP_HEIGHT 310
+static Display *popup_display;
+static Window popup_window;
+static GC popup_gc;
+
+void draw_popup_window( ){
+	if(!popup_window_open) return;
+
+	// save state
+	Window orig_window = window.window;
+	GC orig_gc = window.gc;
+	window.window = popup_window;
+	window.gc = popup_gc;
+
+	int w, h;
+	if(cfg_popup_fullscreen){ w = window.width; h = window.height; } else { w = POPUP_WIDTH; h = POPUP_HEIGHT; }
+
+	drawing_popup_window = 1;
+	if(cfg_popup_fullscreen){
+		// background
+		x_set_color(bg_color);
+		x_fill_rectangle(0, 0, w,h);
+		// widgets
+		int cx = w>>1, cy = h>>1;
+		draw_battery_widget(1, cx-(hw_device_locked ? 100 : 200), cy-100);
+		use_minimal_ui = 1;
+		draw_clock_widget(1, cx-(hw_device_locked ? 80 : 200), cy+70);
+	} else {
+		// border
+		x_set_color(line_color);
+		x_fill_rectangle(0, 0, w,h);
+		// background
+		x_set_color(bg_color);
+		x_fill_rectangle(5, 5, w-10,h-10);
+		// widgets
+		if(!hw_device_locked) draw_battery_widget(1, 5, 5);
+		use_minimal_ui = 1;
+		draw_clock_widget(1, (hw_device_locked ? 125 : 5), (hw_device_locked ? 100 : 155));
+	}
+	draw_text("tap to close", 14, line_color, w>>1, (cfg_popup_fullscreen ? h : h-5), ALIGN_CENTER, ALIGN_BOTTOM, 0);
+	drawing_popup_window = use_minimal_ui = 0;
+
+	// restore state
+	window.window = orig_window;
+	window.gc = orig_gc;
+
+	XFlush(popup_display);
+}
+
+void window_open_popup( ){
+	// this window catches the first key press or screen tap used to wake the device when not locked
+
+	if(popup_window_open || !cfg_popup_duration || use_minimal_ui || current_dialog_draw == draw_flashlight || current_dialog_draw == draw_fullsreen_clock)
+		return;
+
+	debug_write("window_open_popup()");
+
+	// open display
+	if((popup_display = XOpenDisplay(NULL)) == 0){
+		error_write("can't open popup display: %s", XDisplayName(NULL));
+		return;
+	}
+
+	// create window
+	{
+		XSetWindowAttributes winattr;
+		int x, y, w, h;
+
+		if(cfg_popup_fullscreen){ x = y = 0; w = window.width; h = window.height; }
+		else { x = (window.width>>1)-(POPUP_WIDTH/2); y = (window.height>>1)-(POPUP_HEIGHT/2)+50; w = POPUP_WIDTH; h = POPUP_HEIGHT; }
+
+		popup_window = XCreateWindow(popup_display, DefaultRootWindow(popup_display), x,y, w,h,
+										0, CopyFromParent, CopyFromParent, CopyFromParent, 0, &winattr);
+		popup_gc = XCreateGC(popup_display, popup_window, 0, NULL);
+		XSetGraphicsExposures(popup_display, popup_gc, False);
+		XSelectInput(popup_display, popup_window, StructureNotifyMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask | KeyReleaseMask | ExposureMask);
+	}
+
+	// set name
+	{
+		XTextProperty text_prop;
+
+		text_prop.value = "ASUI CLOCK";
+		text_prop.encoding = XInternAtom(popup_display, "STRING", False);
+		text_prop.format = 8;
+		text_prop.nitems = strlen(APPNAME);
+		XSetWMName(popup_display, popup_window, &text_prop);
+	}
+
+	// set protocols -- no window decorations
+	{
+		struct s_mwm_hints motif_hints;
+		Atom props[5];
+		Atom motif_atom;
+
+		XSetWMProtocols(popup_display, popup_window, props, 0);
+
+		motif_hints.flags = MWM_DECORATIONS;
+		motif_hints.decorations = 0; // disable window decorations
+		motif_atom = XInternAtom(popup_display, "_MOTIF_WM_HINTS", False);
+		XChangeProperty(popup_display, popup_window, motif_atom, motif_atom, 32, PropModeReplace,
+						(unsigned char *)&motif_hints, sizeof(motif_hints) / 4);
+	}
+
+	// map the window
+	{
+		XEvent e;
+
+		XMapWindow(popup_display, popup_window);
+		for(;;){
+			usleep(50000); // sleep for 50ms
+			XNextEvent(popup_display, &e);
+			if(e.type == MapNotify) break;
+			debug_write("    waiting for MapNotify event on popup window...");
+		}
+	}
+
+	draw_popup_window();
+
+	popup_fd = ConnectionNumber(popup_display);
+	popup_window_open = 1;
+	popup_window_open_time = current_time;
+	dbus_send_wakeup(_WAKEUP_REFRESH_); // add popup_fd to main loop
+
+	debug_write("    popup window ready");
+}
+
+void window_close_popup( ){
+	if(popup_window_open){
+		debug_write("window_close_popup()");
+
+		popup_fd = 0;
+		XUnmapWindow(popup_display, popup_window);
+		XFreeGC(popup_display, popup_gc);
+		XDestroyWindow(popup_display, popup_window);
+
+		XFlush(popup_display);
+
+		XSetCloseDownMode(popup_display, DestroyAll);
+		XCloseDisplay(popup_display);
+
+		popup_window_open = 0;
+		popup_window_close_time = current_time;
+	}
+}
+
+
+void window_popup_events( ){
+	if(!popup_window_open) return;
+
+	debug_write("any X events for popup window?");
+	while(XPending(popup_display)){
+		XEvent e;
+
+		XNextEvent(popup_display, &e);
+		debug_write("    popup event %d", e.type);
+
+		// process events
+		switch(e.type){
+		case ButtonPress:
+			// ignore taps from the popup window
+			break;
+		case ButtonRelease:
+			// close the popup window
+			window_close_popup();
+			return; // get out of this loop or it will segfault
+		case KeyPress:
+			// ignore keys from the popup window
+			break;
+		case KeyRelease:
+			if(cfg_power_unlock_key && hw_screen.locked){
+
+				if(e.xkey.keycode == power_unlock_keys[cfg_power_unlock_key])
+					mce_unlock_screen();
+				else
+					notify_power_unlock_key();
+
+			} else {
+
+				// close the popup window
+				window_close_popup();
+				return; // get out of this loop or it will segfault
+
+			}
+			break;
+		case Expose:			// 12
+			// force a window redraw for systems without save-unders
+			if(e.xexpose.count == 0) draw_popup_window();
+
+// TODO: rotating from landscape <-> portrait forces a redraw and then an expose event forces another redraw
+
+			break;
+		case NoExpose:			// 14
+		case DestroyNotify:		// 17
+		case UnmapNotify:		// 18
+		case MapNotify:			// 19
+		case ReparentNotify:	// 21
+		case ConfigureNotify:	// 22
+		case ClientMessage:		// 33
+		case MappingNotify:		// 34
+			// ignore
+			break;
+		default:
+			debug_write("unknown popup event %d", e.type);
+		}
+	}
+}
